const responseHelper = require("../Helper/ResponseHelper");
const queryHelper = require("../Helper/QueryHelper");
const dbHelper = require("../Helper/DbHelper");

const STATUS = {IN_PROGRESS: 'InProgress', SUSPEND: 'Suspend'}

const ASSESSMENT_TELORANCE = 86400000 * 30

function create(userId, inputs) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('INSERT INTO assessments (u_id) VALUES (?)', [userId]))
        .then(data => dbHelper.getInsertId(data, responseHelper.Treatment.notFound))
        .then(assesId => update(assesId, inputs))
}

function validateAssessmentTime(userId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('UPDATE assessments set status = ? where (NOW() - time) > ? AND u_id = ? AND status = ?',
            [STATUS.SUSPEND, ASSESSMENT_TELORANCE, userId, STATUS.IN_PROGRESS]))
        .then(data => dbHelper.mustUpdate(data))
}

function suspendAllAssessment(userId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('UPDATE assessments set status = ? where u_id = ? AND status = ?', [STATUS.SUSPEND, userId, STATUS.IN_PROGRESS]))
        .then(data => dbHelper.mustUpdate(data))
}

function update(assessId, symbols) {
    let {queries, inputs} = queryHelper(symbols, '=', ',', '')

    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query(`UPDATE assessments
                            SET ${queries}
                            where id = ${assessId}`, [...inputs]))
        .then(data => dbHelper.mustUpdate(data))
        .then(() => assessId)
}

function insertOrUpdate(userId, userStep, inputs) {
    return validateAssessmentId(userId)
        .then(zeroOrAssessment => {
            inputs.lastUpdateUserStep = userStep;
            return zeroOrAssessment ?
                update(zeroOrAssessment.id, inputs) :
                create(userId, inputs)
        })
}

function getAssessmentByUserId(userId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('SELECT * FROM assessments WHERE u_id = ? AND status = ? limit 1', [userId, STATUS.IN_PROGRESS]))
        .then(data => dbHelper.getSingleton(data, responseHelper.Assessment.notFound))
}

function getAssessmentById(assessId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('SELECT * FROM assessments WHERE id = ?  limit 1', [assessId]))
        .then(data => dbHelper.getSingleton(data, responseHelper.Assessment.notFound))
}

async function validateAssessmentId(userId) {
    try {
        return  await getAssessmentByUserId(userId)
    } catch (e) {
        return 0
    }
}

function getAssessmentAnswer(userId, step) {
    return getAssessmentAnswers(userId, step).then(answers => answers ? answers[step] : undefined)
}

function getAssessmentAnswers(userId, steps) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query(`SELECT ${steps}
                            FROM assessments
                            WHERE u_id = ?
                              AND status = ?
                            LIMIT 1`,
            [userId, STATUS.IN_PROGRESS]))
        .then(dbHelper.getSingleton)
}

function fillGeneralAnswers(userStep, userId) {
    const questionHelper = require("../Helper/QuestionHelper");
    const element = questionHelper[userStep];
    const childrenIds = element.children.map(element => element.id).join(",")

    return getAssessmentAnswers(childrenIds, userId)
        .then(answers => {
            for (const children of element.children) {
                delete children.value;
                for (const answersKey in answers)
                    if (Boolean(answers[answersKey]) && children['id'] === answersKey)
                        children.value = convertToNumber(answers[answersKey])
            }
            return element
        })

}

function fillDedicatedAnswer(userStep, userId) {

    const questionHelper = require("../Helper/QuestionHelper");
    const element = questionHelper[userStep];
    return getAssessmentAnswer(userStep, userId)
        .then(answer => {
            delete element.value
            answer && (element.value = answer)
            return element
        })
}

function convertToNumber(inputValue) {
    let number = parseFloat(inputValue);
    if (!isNaN(number)) return number;
    else return inputValue;
}

function getLastUserStep(userId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('SELECT lastUpdateUserStep FROM assessments WHERE u_id = ? AND status = ? ORDER BY id DESC LIMIT 1',
            [userId, STATUS.IN_PROGRESS]))
        .then(dbHelper.getSingleton)
        .then(data => data?.lastUpdateUserStep)
}

module.exports = {
    update, STATUS, validateAssessmentTime, getAssessment: getAssessmentByUserId, getAssessmentAnswer,
    insertOrUpdate, getLastUserStep, getAssessmentsById: getAssessmentById, fillGeneralAnswers, fillDedicatedAnswer,
    suspendAllAssessment
};