const STARTER_ID = 'p1';
const redFlagController = require("./FormController/RedFlagController");
const util = require("util");
const Gender = Object.freeze({women: "women", men: "men", unknown: "unknown"});

const Side = Object.freeze({
    left: 0,
    right: 1,
    center: 2,
    both: 3,
    unknown: -1
})

const left = ['o7', 'o8', 'o9', 'o10', 'o11', 'o16', 'o17', 'o18']
const right = ['o1', 'o3', 'o4', 'o5', 'o6', 'o13', 'o14', 'o15']
const center = ['o2', 'o12']

const FIELDS = {name: 'p1m1'}

function getMaxPainSide(assessment) {
    const painSides = assessment["p2m1"].toString().split(",");

    let leftIndex = 0, rightIndex = 0, centerIndex = 0;
    for (const painSide of painSides) {
        if (left.includes(painSide)) leftIndex++;
        else if (right.includes(painSide)) rightIndex++;
        else if (center.includes(painSide)) centerIndex++;
    }

    const indexArray = [leftIndex, rightIndex, centerIndex]
    const enumArray = [Side.left, Side.right, Side.center]
    return enumArray[indexArray.indexOf(Math.max(...indexArray))];
}

function havePainInSide(assessment, ...sides) {
    const painSides = {
        [Side.right]: (userPainSides) => right.some(aria => userPainSides.includes(aria)),
        [Side.left]: (userPainSides) => left.some(aria => userPainSides.includes(aria)),
        [Side.center]: (userPainSides) => center.some(aria => userPainSides.includes(aria)),
        [Side.both]: (userPainSides) => painSides[Side.right](userPainSides) && painSides[Side.left](userPainSides),
        [Side.unknown]: () => false,
    }

    const userPainSides = assessment["p2m1"].toString().split(",");

    return sides.some(side => painSides[side](userPainSides));
}

function findGeneralTreatment(assessment) {
    if (assessment.p4m3 === "1" || assessment.p4m4 === "1" || assessment.p4m5 === "1" || assessment.p4m7 === "1")
        return "GT-Ext"
    else if (assessment.p4m10 === "1")
        return "GT-Flx"
    else if (assessment.p4m2 === "1" && assessment.p4m6 === "0")
        return "GT-Flx"
    else
        return "GT-Ext"

}

function findAdvanceTreatment(currentTreatment) {
    switch (currentTreatment) {
        //General
        case 'GT-Ext':
            return 'DT-Ext-D-E'
        case 'GT-Lat':
            return 'DT-Lat-S'
        case 'GT-Flx':
            return 'DT-Flx-D-E'

        //Static
        case 'DT-Ext-S':
            return 'DT-Ext-D-E'
        case 'DT-Lat-S':
            return 'DT-Lat-D-E'

        //Dynamic Simple
        case 'DT-Ext-D-E':
            return 'DT-Ext-D-A'
        case 'DT-Lat-D-E':
            return 'DT-Lat-D-A'
        case 'DT-Flx-D-E':
            return 'DT-Flx-D-A'

        //Dynamic Advance
        case 'DT-Ext-D-A':
            return 'AC'
        case 'DT-Lat-D-A':
            return 'AC'
        case 'DT-Flx-D-A':
            return 'AC'

        default:
            return undefined
    }
}

function findIncrementTreatment(currentTreatment) {
    switch (currentTreatment) {
        //General
        /*        case 'GT-Ext':
                    return 'd3'
                case 'GT-Lat':
                    return 'd3'
                case 'GT-Flx':
                    return 'd3'*/

        //Static
        case 'DT-Ext-S':
            return 'DT-Ext-D-E'
        case 'DT-Lat-S':
            return 'DT-Lat-D-E'

        //Dynamic
        case 'DT-Ext-D-E':
            return 'NTV'
        /*case 'DT-Lat-D-E':
            return 'DT-Lat-D-A'*/
        case 'DT-Flx-D-E':
            return 'NTV'

        default:
            return undefined
    }
}

function findBackGeneralStep(currentStep) {
    switch (currentStep) {
        case "p1":
            return undefined;
        case "p2":
            return 'p1';
        case "PW":
            return 'p1';
        case "p3":
            return 'p2';
        case "p4":
            return "p5"
        case "p5":
            return 'p3';
        case "p6":
            return 'p4';
    }
}

function findNextGeneralStep(assessment, currentStep) {
    switch (currentStep) {
        case "p1":
            if (getGender(assessment) === Gender.women)
                return 'PW';
            else
                return 'p2';
        case "PW":
            return 'p2';
        case "p2":
            return 'p3';
        case "p3":
            return 'p5';
        case "p5":
            if (redFlagController.isRedFlag(assessment))
                return "RF"
            else
                return 'p4';
        case "p4":
            return "p6"
        case "p6":
            return findGeneralTreatment(assessment)
    }
}

function findBackDedicateStep(assessment, currentStep) {
    switch (currentStep) {
        case "d3":
            return undefined
        case "d4":
            return 'd3';
        case "d5":
            return 'd4'
        case "d6":
            return 'd5'
        case "d7":
            return 'd6';
        case "d8":
            return 'd7'
        case "d9":
            return 'd8';
        case "d10":
            return havePainInSide(assessment, Side.left) ? 'd7' : 'd9'
        case "d11":
            return 'd10';
        case "d12":
            return havePainInSide(assessment, Side.left) ?
                'd11' : havePainInSide(assessment, Side.right) ?
                    'd9' : 'd7'
        case "d13":
            return 'd12';
        case "d14":
            return 'd13'
        case "d15":
            return 'd14';
        case "d16":
            return 'd15'
        case "d17":
            return 'd16';
        case "d18":
            return 'd17'
        case "d19":
            return 'd18';
    }
}

function findNextDedicateStep(assessment, currentStep) {
    const currentStepValue = assessment[currentStep]
    switch (currentStep) {
        case "d3":
            switch (currentStepValue) {
                case 'd3o2':
                    return "DT-Ext-S"
                case 'd3o3':
                    return "DT-Lat-S"
                default :
                    return 'd4'
            }
        case "d4":
            switch (currentStepValue) {
                case 'd4o1':
                    return "DT-Ext-D-E"
                case 'd4o3':
                    return "d6"
                default :
                    return 'd5'
            }
        case "d5":
            if (currentStepValue === 'd5o1')
                return "DT-Ext-D-E"
            else
                return 'd6';
        case "d6":
            switch (currentStepValue) {
                case 'd6o1':
                    return "DT-Ext-D-E"
                case 'd6o3':
                    if (havePainInSide(assessment, Side.center,Side.both))
                        return "d8"
                    else if (havePainInSide(assessment, Side.left))
                        return "d10"
                    else
                        return "d8"
                default :
                    return 'd7';
            }
        case "d7":
            switch (currentStepValue) {
                case 'd7o1':
                    return "DT-Flx-D-E"
                default :
                    if (havePainInSide(assessment,Side.left))
                        return "d10"
                    else
                        return 'd8';
            }

        case "d8":
            switch (currentStepValue) {
                case 'd8o1':
                    return "DT-Lat-D-E-2"
                case 'd8o3':
                    if (havePainInSide(assessment, Side.center,Side.both))
                        return "d10"
                    else if (havePainInSide(assessment, Side.right))
                        return "d12"
                    else
                        return "d10"
                default :
                    return 'd9';
            }


        case "d9":
            switch (currentStepValue) {
                case 'd9o1':
                    return "DT-Lat-D-E-2"
                default :
                    if (havePainInSide(assessment,Side.right))
                        return "d12"
                    else
                        return 'd10';
            }

        case "d10":
            switch (currentStepValue) {
                case 'd10o1':
                    return "DT-Lat-D-E-2"
                case 'd10o3':
                    return "d12"
                default :
                    return 'd11';
            }


        case "d11":
            if (currentStepValue === 'd11o1')
                return "DT-Lat-D-E-2"
            else
                return 'd12';

        case "d12":
            switch (currentStepValue) {
                case 'd12o1':
                    return "DT-Flx-D-E"
                case 'd12o3':
                    return "d14"
                default :
                    return 'd13';
            }

        case "d13":
            if (currentStepValue === 'd13o1')
                return "DT-Flx-D-E"
            else
                return 'd14';

        case "d14":
            switch (currentStepValue) {
                case 'd14o1':
                    return "DT-Ext-D-E"
                case 'd14o3':
                    if (havePainInSide(assessment, Side.center,Side.both))
                        return "d16"
                    else if (havePainInSide(assessment, Side.right))
                        return "d18"
                    else
                        return "d16"
                default :
                    return 'd15';
            }


        case "d15":
            switch (currentStepValue) {
                case 'd15o1':
                    return "DT-Ext-D-E"
                default :
                    if (havePainInSide(assessment,Side.right))
                        return "d18"
                    else
                        return 'd16';
            }

        case "d16":
            switch (currentStepValue) {
                case 'd16o1':
                    return "DT-Lat-D-E-3"
                case 'd16o3':
                    if (havePainInSide(assessment, Side.center,Side.both))
                        return "d18"
                    else if (havePainInSide(assessment, Side.left))
                        return "NTV"
                    else
                        return "d18"
                default :
                    return 'd17';
            }
        case "d17":
            switch (currentStepValue) {
                case 'd17o1':
                    return "DT-Lat-D-E-3"
                default :
                    if (havePainInSide(assessment,Side.left))
                        return "NTV"
                    else
                        return 'd18';
            }
        case "d18":
            if (currentStepValue === 'd18o1')
                return "DT-Lat-D-E-3"
            return 'd19';
        case "d19":
            if (currentStepValue === "d19o1")
                return "DT-Lat-D-E-3"
            else
                return "NTV"
    }
}

function getGender(assessment) {
    switch (assessment["p1m6"]) {
        case "p1m6o1":
            return Gender.women;
        case "p1m6o2":
            return Gender.men;
        default:
            return Gender.unknown;
    }
}

function getPain(assessment) {
    const pains = []
    if (assessment["p3m2"].toString().includes("p3m2o3")) pains.push("ایجاد درد به علت ضربه )در ماه اخیر(")
    if (assessment["p3m2"].toString().includes("p3m2o4")) pains.push("ایجاد درد به علت تصادف )در ماه اخیر(")
    if (assessment["p3m2"].toString().includes("p3m2o5")) pains.push("ایجاد درد به علت افتادن )در ماه اخیر(")
    if (assessment["p5m1"] === "1") pains.push("اختلال در دفع یا بی اختیاری ادرار")
    if (assessment["p5m2"] === "1") pains.push("احساس بی حسی در ناحیه کف لگن )پرینه یا شرمگاهی(")
    if (assessment["p5m3"] === "1") pains.push("احساس اختلال در راه رفتن")
    if (assessment["p5m4"] === "1") pains.push("بارداری")
    if (assessment["p5m5"] === "1") pains.push("ضعف جدی هنگام راه رفتن روی پاشنه")
    if (assessment["p5m6"] === "1") pains.push("ضعف جدی هنگام راه رفتن روی پنجه")
    if (assessment["p5m7"] === "1") pains.push("احساس کاهش وزن ناخواسته یا بی دلیل )بیشتر از 4 کلیوگرم در ماه(")
    if (assessment["p5m8"] === "1") pains.push("سابقه جراحی فیوژن")
    if (assessment["p5m13"] === "1") pains.push("سابقه بدخیمی‌یا تومور در ناحیه کمری-لگنی")
    return pains;
}

function getName(assessment) {
    return assessment[FIELDS.name];
}

function getEmail(assessment) {
    return assessment["p1m8"];
}

function getNameWithGender(assessment) {
    const gender = {women: 'سرکار خانم %s', men: 'جناب آقای %s', unknown: '%s'}
    let name = getName(assessment);
    const genderIndex = getGender(assessment);
    const nameWithGenderText = gender[genderIndex];
    name = util.format(nameWithGenderText, name)
    return name;
}

module.exports = {
    getNameWithGender, findAdvanceTreatment,
    getName, getPain, getEmail, findNextDedicateStep, findNextGeneralStep, findBackGeneralStep, findBackDedicateStep,
    FIELDS, STARTER_ID, findIncrementTreatment
};
