const util = require('util');
const relationController = require("../Controller/RelationController");
const fs = require("fs");
const path = require("path");
const PDFDocument = require('pdfkit');
const jalaaliHelper = require('./JalaliHelper');


function create(pId, assessment) {
    return new Promise(resolve => {
        const inputFile = path.resolve(__dirname, '../public/ReferenceLetterEmpty.jpg')
        const outputPath = path.resolve(__dirname, util.format(process.env.PDF_PRIVATE_PATH, pId));
        const fontFilePath = path.resolve(__dirname, '../public/font.ttf');

        const doc = new PDFDocument({size: 'A5'});
        doc.pipe(fs.createWriteStream(outputPath));

        doc.font(fontFilePath).fontSize(10)

        const persianDateText = jalaaliHelper.toJalaliDateText(new Date())

        const name = relationController.getNameWithGender(assessment);

        const pains = relationController.getPain(assessment)
        let painsText = pains.join(" و ")

        let text = `احتراما بدين وسيله ${name} با اعلام ${painsText} جهت مشاوره درمانی به حضورتان معرفی می‌شود.`

        doc.image(inputFile, 0, 0, {width: 420})

        doc.text(text, 50, 200, {align: 'right', width: 300, paragraphGap: 30, features: ['rtla']});

        doc.fillColor('white').text(persianDateText, 85, 66)

        doc.end();

        resolve(util.format(process.env.PDF_PUBLIC_PATH, pId))
    })
}

module.exports = {create}