const express = require('express');
const router = express.Router();
require('dotenv').config()

const userController = require('../Controller/UserController')
const treatmentController = require('../Controller/TreatmentController')
const paymentController = require('../Controller/PaymentController')
const assessmentsController = require("../Controller/AssessmentsController");
const relationController = require("../Controller/RelationController");
const sMSController = require("../Controller/SMSController");
const redFlagController = require("../Controller/FormController/RedFlagController");

const authHelper = require('../Helper/AuthHelper')
const responseHelper = require('../Helper/ResponseHelper')
const validatorHelper = require('../Helper/ValidatorHelper')

router.get('/', (req, res) => res.send('OK'));

router.post("/otp", (req, res) => {
    const {pId} = req.body

    if (!validatorHelper.validatePhoneNumber(pId)) return onRejected(responseHelper.User.notVerified)

    userController.verifyPId(pId)
        .then(userId => createOTP(pId, userId))
        .catch(error => {
            if (error === responseHelper.User.notFound)
                userController.creatUser(pId)
                    .then(userId => createOTP(pId, userId))
                    .catch(onRejected);
            else
                onRejected(error)

        });

    function createOTP(pId, userId) {
        return userController.generateNewOTP(userId)
            .then(otp => sMSController.otp(pId, otp))
            .then(() => res.json({success: true, message: responseHelper.OTP.sent, data: {accessToken: authHelper.getAccessToken(userId)}}))
    }

    function onRejected(error) {
        res.json({success: false, message: error, data: {}})
    }

});

router.use(authHelper.auth)

router.post("/verifyOtp", (req, res) => {
    const {otp} = req.body;
    const userId = req.userId;

    userController.verifyOTP(userId, otp)
        .then(() => userState(req, res))
        .catch(onRejected)

    function onRejected(error) {
        res.json({success: false, message: error, data: {}})
    }

});

router.post("/userState", (req, res) => userState(req, res));

router.post("/resetAssessment", (req, res) => {
    const userId = req.userId;

    assessmentsController.suspendAllAssessment(userId)
        .then(suspendCount => userController.updateStep(userId, 'p1'))
        .then(() => res.json({success: true, message: "ارزیابی‌های شما ریست شد.", data: {}}))
        .catch(onRejected)

    function onRejected(error) {
        res.json({success: false, message: error, data: {}})
    }

})

function userState(req, res) {
    const userId = req.userId;
    const {accessToken} = req.body;
    let userStep, registeredLabel;

    assessmentsController.validateAssessmentTime(userId)
        .then(affectedRows =>
            Promise.all([
                affectedRows ? userController.updateStep(userId, relationController.STARTER_ID) : userController.getUserStep(userId),
                assessmentsController.getAssessmentAnswer(userId, relationController.FIELDS.name)
            ]))
        .then(results => {
            userStep = results[0];
            registeredLabel = results[1] || 'کاربر جدید';
            return separateStep();
        })
        .catch(onRejected)

    function onRejected(error) {
        res.json({success: false, message: error, data: {}})
    }

    function separateStep() {
        if (userStep.startsWith('p') || userStep.startsWith('PW'))
            return assessmentsController.fillGeneralAnswers(userStep, userId).then(sendResponse)
        else if (userStep.startsWith('GT') || userStep.startsWith('DT'))
            return treatmentController.prepareTreatments(userId, userStep).then(sendResponse)
        else if (userStep.startsWith('d'))
            return assessmentsController.fillDedicatedAnswer(userStep, userId).then(sendResponse)
        else if (userStep.startsWith('AC'))
            userController.getUserPId(userId)
                .then(redFlagController.getAllCuredForm)
                .then(sendResponse)
        else if (userStep.startsWith('RF'))
            userController.getUserPId(userId)
                .then(redFlagController.getRedFlagForm)
                .then(sendResponse)
        else if (userStep.startsWith('P'))
            userController.getUserPId(userId)
                .then(pId => paymentController.preparePayment(userId, pId, registeredLabel, accessToken))
                .then(paymentLink => paymentController.getPaymentForm(paymentLink))
                .then(sendResponse)
        else if (userStep.startsWith('NTV'))
            redFlagController.getNeedToVisitForm().then(sendResponse)
        else res.json({
                success: false,
                message: responseHelper.Assessment.notFound,
                data: {}
            })

        function sendResponse(currentForm) {
            currentForm.registeredLabel = registeredLabel;
            res.json({
                success: true, message: "",
                data: currentForm
            })
        }
    }
}

module.exports = router;